@@ -43,18 +43,45 @@ class KombuManager(PubSubManager):  # pragma: no cover
             raise RuntimeError('Kombu package is not installed '
                                '(Run "pip install kombu" in your '
                                'virtualenv).')
-        self.kombu = kombu.Connection(url)
-        self.exchange = kombu.Exchange(channel, type='fanout', durable=False)
-        self.queue = kombu.Queue(str(uuid.uuid4()), self.exchange)
-        super(KombuManager, self).__init__(channel=channel,
-                                           write_only=write_only)
+        super(KombuManager, self).__init__(channel=channel)
+        self.url = url
+        self.producer = self._producer()
+
+    def initialize(self):
+        super(KombuManager, self).initialize()
+
+        monkey_patched = True
+        if self.server.async_mode == 'eventlet':
+            from eventlet.patcher import is_monkey_patched
+            monkey_patched = is_monkey_patched('socket')
+        elif 'gevent' in self.server.async_mode:
+            from gevent.monkey import is_module_patched
+            monkey_patched = is_module_patched('socket')
+        if not monkey_patched:
+            raise RuntimeError(
+                'Redis requires a monkey patched socket library to work '
+                'with ' + self.server.async_mode)
+
+    def _connection(self):
+        return kombu.Connection(self.url)
+
+    def _exchange(self):
+        return kombu.Exchange(self.channel, type='fanout', durable=False)
+
+    def _queue(self):
+        queue_name = 'flask-socketio.' + str(uuid.uuid4())
+        return kombu.Queue(queue_name, self._exchange(),
+                           queue_arguments={'x-expires': 300000})
+
+    def _producer(self):
+        return self._connection().Producer(exchange=self._exchange())
 
     def _publish(self, data):
-        with self.kombu.SimpleQueue(self.queue) as queue:
-            queue.put(pickle.dumps(data))
+        self.producer.publish(pickle.dumps(data))
 
     def _listen(self):
-        with self.kombu.SimpleQueue(self.queue) as queue:
+        reader_queue = self._queue()
+        with self._connection().SimpleQueue(reader_queue) as queue:
             while True:
                 message = queue.get(block=True)
                 message.ack()
